from cpl_core.configuration import ConfigurationABC
from cpl_discord.events import OnReadyABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.environment_variables import MAINTENANCE
from bot_core.logging.event_logger import EventLogger


class CoreExtensionOnReadyEvent(OnReadyABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: EventLogger,
        bot: DiscordBotServiceABC,
        client_utils: ClientUtilsABC,
        t: TranslatePipe,
    ):
        OnReadyABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._client_utils = client_utils
        self._t = t

        self._logger.info(__name__, f"Module {type(self)} loaded")

    async def on_ready(self):
        self._logger.debug(__name__, f"Module {type(self)} started")
        await self._client_utils.set_maintenance_mode(self._config.get_configuration(MAINTENANCE))
        self._logger.trace(__name__, f"Module {type(self)} stopped")
