from cpl_core.configuration import ConfigurationABC
from cpl_discord.command import DiscordCommandABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger
from bot_core.service.message_service import MessageService
from bot_data.model.server_config import ServerConfig
from modules.base.service.event_service import EventService


class ScheduledEventsCommand(DiscordCommandABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: CommandLogger,
        events: EventService,
        message_service: MessageService,
        t: TranslatePipe,
    ):
        DiscordCommandABC.__init__(self)

        self._config = config
        self._logger = logger
        self._events = events
        self._message_service = message_service
        self._t = t

        self._logger.trace(__name__, f"Loaded command service: {type(self).__name__}")

    @commands.hybrid_group(name="scheduled-events")
    @commands.guild_only()
    async def scheduled_events(self, ctx: Context):
        pass

    @scheduled_events.command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def reload(self, ctx: Context):
        settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{ctx.guild.id}")
        if not FeatureFlagsSettings.get_flag_from_dict(settings.feature_flags, FeatureFlagsEnum.scheduled_events):
            self._logger.debug(__name__, f"Skipping scheduled event check for {ctx.guild.id}. Feature is disabled.")
            await self._message_service.send_ctx_msg(ctx, self._t.transform("common.feature_not_activated"))
            return

        self._logger.debug(__name__, "Running scheduled-events reload")
        try:
            await self._events.check_and_create_scheduled_events(ctx.guild)
        except Exception as e:
            self._logger.error(__name__, f"Reloading scheduled events failed", e)
