from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import tasks

from bot_core.abc.task_abc import TaskABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_core.logging.task_logger import TaskLogger
from bot_core.service.message_service import MessageService
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.server_config import ServerConfig
from modules.base.service.event_service import EventService


class ScheduledEventsWatcher(TaskABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: TaskLogger,
        bot: DiscordBotServiceABC,
        db: DatabaseContextABC,
        servers: ServerRepositoryABC,
        events: EventService,
        message_service: MessageService,
        t: TranslatePipe,
    ):
        TaskABC.__init__(self)

        self._config = config
        self._logger = logger
        self._bot = bot
        self._db = db
        self._servers = servers
        self._events = events
        self._message_service = message_service
        self._t = t

        if not self._is_maintenance():
            self.watch.start()

    @tasks.loop(hours=3, reconnect=True)
    async def watch(self):
        self._logger.info(__name__, "Watching scheduled events")
        try:
            for guild in self._bot.guilds:
                settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{guild.id}")
                if not FeatureFlagsSettings.get_flag_from_dict(
                    settings.feature_flags, FeatureFlagsEnum.scheduled_events
                ):
                    self._logger.debug(__name__, f"Skipping scheduled event check for {guild.id}. Feature is disabled.")
                    continue

                await self._events.check_and_create_scheduled_events(guild)
        except Exception as e:
            self._logger.error(__name__, f"Watching scheduled events failed", e)

    @watch.before_loop
    async def wait(self):
        await self._wait_until_ready()
